'''
Script that merges 2 Excel data files. Used to add the PIDs and session number information to certain files. 

Author: Vlad Iftimescu
Date: January 2024

Previously-used input file pairs:
(User_Needs_Quotations.xlsx, Peer_Review_Quotations_Anon.xlsx) - added the codes of the second coder to the User_Needs_Quotations.xlsx filtered quotation file
(User_Needs_Quotations.xlsx, FrankensteinDataPIDs.xlsx) - added the PIDs and activity session numbers to the User_Needs_Quotations.xlsx. filtered quotation file

Output: User_Needs_Quotations.xlsx
'''

import pandas as pd

# Method that transforms the quotation string such that each quotation is uniquely identifiable
def modified_content(s):
    if isinstance(s, str):
        return ''.join(filter(str.isalpha, s)) # Only keep the alphabetic characters in the string
    else:
        return ''

# Take the paths for the input Excel files
FILE_PATH_1 = "User_Needs_Quotations.xlsx"
FILE_PATH_2 = "FrankensteinDataPIDs.xlsx"

# Retrieve the active worksheets from the Excel files
file_1 = pd.read_excel(FILE_PATH_1, "Quotation Manager")
file_2 = pd.read_excel(FILE_PATH_2, "Sheet1")

# Make sure the 2 files are sorted by quotation strings
file_1 = file_1.sort_values(by="Quotation Content").reset_index(drop=True)
file_2 = file_2.sort_values(by="activity_experience_slot").reset_index(drop=True)

# For each file, apply the function to the quotation column and create an auxiliary column
file_1['ModifiedContent'] = file_1['Quotation Content'].apply(modified_content)
file_2['ModifiedContent'] = file_2['activity_experience_slot'].apply(modified_content)

result = pd.merge(file_1, file_2, on="ModifiedContent", how="left") # Perform the merge operation

# Remove rows where the specified column is empty
result = result.dropna(subset=['Quotation Content'], how='all')

# Drop duplicates based on specified columns
result = result.drop_duplicates(subset=['rand_id', 'session_num'], keep='first')

# Remove the auxiliary column
result = result.drop(columns=['ModifiedContent'])

# Save the merged file as a new Excel file
result.to_excel("User_Needs_Quotations.xlsx", index=False)
