'''
Calculates the Cohen's Kappa values for all codes that were used for the thematic analysis.
The script assumes that the quotations in the 2 files have the same order.

Author: Vlad Iftimescu
Date: January 2024

Input: Research_Project_Final_Quotations_Anon.xlsx, Peer_Review_Quotations_Anon.xlsx
Output: List of Cohen's Kappa values for each thematic analysis code
'''

import openpyxl
import numpy as np
import pandas as pd
from sklearn.metrics import cohen_kappa_score

# File paths for the Excel input files
FILE_1_PATH = "Research_Project_Final_Quotations_Anon.xlsx"
FILE_2_PATH = "Peer_Review_Quotations_Anon.xlsx"

# List of all codes used in the thematic analysis
UNIQUE_CODES = ['Behavior change: Compulsive behaviors', 'Behavior change: Coping strategies', 'Behavior change: Habits', 'Behavior change: Lifestyle changes', 'Behavior change: Obstacles', 'Desire: Cravings', 'Desire: Desire for change', 'Effort: All of activity', 'Effort: Nothing', 'Effort: Part of activity', 'Effort: Related to activity', 'Emotions: Anxiety', 'Emotions: Depression', 'Emotions: Disinterest', 'Emotions: Dislike', 'Emotions: Dissapointment', 'Emotions: Frustration', 'Emotions: Happiness', 'Emotions: Ineffectiveness', 'Emotions: Interest', 'Emotions: Kindness', 'Emotions: Motivation', 'Emotions: Uncertainty', 'Financial: Costs of living', 'Financial: Costs of nicotine replacements', 'Financial: Costs of smoking', 'General factors: Irrelevant to study', 'General factors: Lack of confidence', 'General factors: Lack of involvement', 'General factors: Lack of privacy', 'General factors: Superficial analysis', 'General factors: Visualization', 'Health: Health awareness', 'Health: Health benefits', 'Health: Health concerns', 'Health: Mental health', 'Health: Weight', 'Human experience: Alcohol', 'Human experience: Caffeine', 'Human experience: Excuses', 'Human experience: Food', 'Human experience: Forgetfulness', 'Human experience: Impact on others', 'Human experience: Inspiration', 'Human experience: Remembrance', 'Personal empowerment: Goal setting', 'Personal empowerment: Knowledge enhancement', 'Personal empowerment: Positive attitude', 'Personal empowerment: Self-confidence', 'Personal empowerment: Self-discipline', 'Personal empowerment: Self-improvement', 'Personal empowerment: Self-reflection', 'Smoking cessation: Addiction', 'Smoking cessation: Consequences of smoking', 'Smoking cessation: Impact of quitting smoking', 'Smoking cessation: Nicotine replacement', 'Smoking cessation: Physical effects of smoking', 'Smoking cessation: Smell of smoke', 'Smoking cessation: Smoking alternatives', 'Smoking cessation: Triggers', 'Well-being: Alternative therapy', 'Well-being: Busyness', 'Well-being: Distraction', 'Well-being: External support', 'Well-being: Physical activity', 'Well-being: Physical limitations', 'Well-being: Relaxation', 'Well-being: Sleep quality', 'Well-being: Stress management', 'Well-being: Time constraints', 'Well-being: Time management']

# Class that encapsulates the data of a Quotation
class Quotation:
    def __init__(self, content, codes):
        if content is  None: # in case a None value got into the dataset
            self.content = ''
        else:
            self.content = content
        self.codes = codes.split('\n') # Split the codes into a list

    def __eq__(self, __value: object) -> bool:
        if isinstance(__value, Quotation):
            return self.modified_content() == __value.modified_content() # Check that the unique strings are equal
        return False
    
    def __hash__(self):
        return hash(self.modified_content())
    
    # Method that transforms the content string s.t. each quotation is uniquely identifiable
    def modified_content(self):
        return ''.join(filter(str.isalpha, self.content))[:30]


# Method that creates a coding matrix given a list of Quotations
# - a coding matrix is a matrix that has quotations on its rows and codes on its columns
# - contains 0/1 values denoting the absence/presence of a particular code in a quotation
def create_coding_matrix(quotations):
    coding_matrix = np.zeros((len(quotations), len(UNIQUE_CODES)), dtype=int)
    for i, quotation in enumerate(quotations):
        for j, code in enumerate(UNIQUE_CODES):
            # If a code appears in a quotation, update its corresponding column index in the matrix to 1
            if code in quotation.codes:
                coding_matrix[i, j] = 1
    
    return coding_matrix



# Retrieve the active sheet from each of the 2 files
file_1 = openpyxl.load_workbook(FILE_1_PATH).active
file_2 = openpyxl.load_workbook(FILE_2_PATH).active

# Iterate through all the rows to initialize the Quotation lists for each file
cnt_row = file_1.max_row
quotations_1, quotations_2 = list(), list()
for i in range(2, cnt_row + 1):
    # Get the data from the current row of file 1
    content_1 = str(file_1.cell(row = i, column = 1).value)
    codes_1 = str(file_1.cell(row = i, column = 2).value)
    quotation_1 = Quotation(content_1, codes_1) # Cast the file 1 data to a Quotation

    # Do the same for file 2
    content_2 = str(file_2.cell(row = i, column = 1).value)
    codes_2 = str(file_2.cell(row = i, column = 2).value)
    quotation_2 = Quotation(content_2, codes_2) # Cast the file 2 data to a Quotation

    # Add the Quotations to their respective lists
    quotations_1.append(quotation_1)
    quotations_2.append(quotation_2)

# Create coding matrices for the 2 quotation lists
coding_matrix_1 = np.nan_to_num(create_coding_matrix(quotations_1))
coding_matrix_2 = np.nan_to_num(create_coding_matrix(quotations_2))

kappa_values = [] # Contains the Cohen's Kappa value for each code
for i in range(coding_matrix_1.shape[1]): # Iterate over the columns/labels of the matrices
    labels_1 = coding_matrix_1[:, i]
    labels_2 = coding_matrix_2[:, i]
    # Calculate Kappa value for the current code and append it to the list
    kappa = cohen_kappa_score(labels_1, labels_2)
    kappa_values.append(kappa)
    print("Cohen's Kappa for code", UNIQUE_CODES[i], "=", kappa)


# Create a new Excel file containing the results
df = pd.DataFrame({"Code": UNIQUE_CODES, "Cohen's Kappa value": kappa_values})
output_path = 'cohens_kappa_values.xlsx'
df.to_excel(output_path, index=False) # Write the DataFrame to Excel
