'''
Computes the Spearman correlation between users' highest completed educational levels and the themes.

Author: Vlad Iftimescu
Date: January 2024

Input: pids_codes_characteristics_Anon.xlsx
Output: Figure 3 in the paper
'''

import pandas as pd
import matplotlib.pyplot as plt
from scipy.stats import spearmanr

# Dictionary used to rank education levels by number
EDUCATION_LEVELS = {
    "No formal qualifications": 0,
    "Secondary education (e.g. GED/GCSE)": 1,
    "High school diploma/A-levels": 2,
    "Technical/community college": 3,
    "Undergraduate degree (BA/BSc/other)": 4,
    "Graduate degree (MA/MSc/MPhil/other)": 5,
    "Doctorate degree (PhD/other)": 6
}

# Dictionary used to identify the columns in the frequency matrix by the names in the input file
COLUMNS = {
    "Highest education level completed": 0,
    "Less distressing activities": 1,
    "More interesting activities": 2,
    "More knowledge": 3,
    "More motivation": 4,
    "Confidence-maintaining activities": 5,
    "Reminders": 6,
    "Analytics of tasks": 7,
    "Weight managing tips": 8
}

# Retrieve the DataFrame from the Excel sheet
# The file should have its rows grouped by increasing PID
file = pd.read_excel("pids_codes_characteristics_Anon.xlsx", "Sheet1")

# Map education level strings in the input file to their ranks
file['Highest education level completed'] = file['Highest education level completed'].map(EDUCATION_LEVELS)


# Compute the frequency matrix
dict = {} # Used to keep track of which PIDs we encountered so far
freq = [[0] * 9 for _ in range(121)]
cnt_users = -1 # Used to keep track of the current row in the frequency matrix
for idx, row in file.iterrows():
    # Get the data from the current row of the file
    pid = row["rand_id"]
    education = row["Highest education level completed"]
    codes = row["Codes"].split('\n')

    # If we encounter a user with a new PID
    if pid not in dict:
        dict[pid] = 1
        cnt_users += 1
        freq[cnt_users][COLUMNS["Highest education level completed"]] = education # Add the education level rank to the matrix
    
    # Update the frequency matrix for the current PID
    for code in codes:
        if code[:5] == "Needs" and code[7:] != "Easier/simpler activities": # Check that the code is indeed one of the 8 themes, and ignore the theme that was scrapped
            freq[cnt_users][COLUMNS[code[7:]]] += 1 # Update the frequency of the theme for the current person


df = pd.DataFrame(freq, columns=list(COLUMNS.keys()))

# Compute the correlation matrix and p-values
correlation_matrix = []
p_values = []
# Go through each of the themes (columns 1-9 represent the 8 themes)
for column in df.columns[1:]:
    corr, p_val = spearmanr(df["Highest education level completed"], df[column]) # Calculate the Spearman correlation between the education levels and the theme
    correlation_matrix.append(corr)
    p_values.append(p_val)

print(p_values) # Print the respective P-values which were added to Table 2 in the paper

# Plot the bar chart
plt.bar(df.columns[1:], correlation_matrix)
plt.xlabel('Themes')
plt.ylabel('Correlation with highest education level completed')
plt.xticks(rotation=45, ha='right')
plt.tight_layout()
plt.show()
